﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Ionic.Zip;
using System.Threading;



/*
JamDo .Net, a tool to manage zip files downloaded from Jamendo.

-----------------------------------------------------------------------
LICENSE:

Copyright (c)

C. Hackenberg, 2012. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
   
THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//You better do not learn from this spaghetti code!
namespace JamDoNetTool
{
    public partial class MainForm : Form
    {
        private bool fromContext = false;
        public MainForm()
        {
            InitializeComponent();
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Settings settingsDialog = new Settings();
            settingsDialog.ShowDialog();
        }


 
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About aboutDialog = new About();
            aboutDialog.ShowDialog();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            //Adding Drag and Drop methods
            lBoxFiles.AllowDrop = true;
            lBoxFiles.DragOver += new DragEventHandler(lBoxFilesOnDragOver);
            lBoxFiles.DragDrop += new DragEventHandler(lBoxFilesOnDragDrop);
            //In case the programm is called from the context menu
            string[] contextFiles = Environment.GetCommandLineArgs();
            if (contextFiles.Length > 1)
            {
                this.WindowState = FormWindowState.Minimized;
                Console.WriteLine(contextFiles[0]);
                fromContext = true;
                foreach (string file in contextFiles)
                {
                    this.addJamZip(file);
                }
                backgroundWorker.RunWorkerAsync();
                this.btnStart.Enabled = false;
            }
        }

        /// <summary>
        /// Event handler method for dragging
        /// </summary>
        private void lBoxFilesOnDragOver(object obj, DragEventArgs myDragArgs) 
        {
            if (myDragArgs.Data.GetDataPresent(DataFormats.FileDrop))
            {
                myDragArgs.Effect = DragDropEffects.Move;
            }
            else
            {
                myDragArgs.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Event handler method for actually dropping the file
        /// </summary>
        private void lBoxFilesOnDragDrop(object obj, DragEventArgs myDropArgs)
        {
            if (myDropArgs.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] fileList = (string[])myDropArgs.Data.GetData(DataFormats.FileDrop);
                foreach (string strFile in fileList) 
                {
                    addJamZip(strFile);
                }
            }
        }

        private void addJamZip(string file) 
        {
            //Check the path
            if (Regex.IsMatch(file, "^.* --* Jamendo.*?\\.zip$"))
            {
                lBoxFiles.Items.Add(file);
                this.btnStart.Enabled = true;
            }
        }

        private void emptyFileListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            lBoxFiles.Items.Clear();
        }
        List<string> errors = new List<string>();
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (!backgroundWorker.IsBusy) 
            {
                backgroundWorker.RunWorkerAsync();
                this.btnStart.Enabled = false;
            }            
        }




        private void lBoxFiles_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Delete)
            {
                if (lBoxFiles.Items.Count > 0 && lBoxFiles.SelectedItems.Count > 0)
                {
                    foreach (int iterator in lBoxFiles.SelectedIndices)
                    {
                        lBoxFiles.Items.RemoveAt(iterator);
                    }
                    
                }
            }
        }

        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            if (Directory.Exists(Properties.Settings.Default.TargetDirectory))
            {
                int progressStep = 100 / lBoxFiles.Items.Count;
                foreach (string file in lBoxFiles.Items)
                {
                    if (File.Exists(file))
                    {
                        //Step one: Moving/Copying the file
                        if (Properties.Settings.Default.DeleteZipFiles)
                        {
                            try
                            {
                                File.Move(file, Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileName(file)));
                            }
                            catch 
                            {
                                errors.Add("Error: File " + Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileName(file))+ "is already copied to the target directory.");
                            }
                        }
                        else
                        {
                            try
                            {
                                File.Copy(file, Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileName(file)), false);
                            }
                            catch 
                            {
                                errors.Add("Error: File " + Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileName(file))+ "is already copied to the target directory.");
                            }
                        }
                        Thread.Sleep(100);
                        //Step two: Creating a folder name like the zip package
                        Directory.CreateDirectory(Path.Combine(Properties.Settings.Default.TargetDirectory,Path.GetFileNameWithoutExtension(file)));
                        //Step three: Unpacking the zip file into the created folder
                        try 
                        {
                            using (ZipFile zip = ZipFile.Read(Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileName(file))))
                            {
                                zip.ExtractAll(Path.Combine(Properties.Settings.Default.TargetDirectory,Path.GetFileNameWithoutExtension(file)));
                            }
                            //Updating the ProgressBar
                            backgroundWorker.ReportProgress(progressStep);
                        }
                        catch 
                        {
                            errors.Add("Error: Unable to extract zip package "+Path.GetFileName(file)+" .");
                        }
                        //Step four: Delete the package in the target folder
                        Thread.Sleep(100);
                        try
                        {
                            File.Delete(Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileName(file)));
                        }
                        catch 
                        {
                            errors.Add("Error: Unable to delete zip package"+Path.GetFileName(file)+" .");
                        }
                        //Step five: Rename the new folder
                        if (Properties.Settings.Default.RenamePackages)
                        {
                            Thread.Sleep(100);
                            string newPath = Regex.Replace(Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileNameWithoutExtension(file)), "(\\.*?) --* Jamendo.*", "$1");
                            try
                            {
                                Directory.Move(Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileNameWithoutExtension(file)), newPath);
                            }
                            catch 
                            {
                                errors.Add("Error: Unable to rename " + Path.Combine(Properties.Settings.Default.TargetDirectory, Path.GetFileNameWithoutExtension(file)) + " .");
                            }
                        }
                    }
                    else
                    {
                        errors.Add("Error: File " + file + " not found.");
                    }
                }
            }
            else
            {
                errors.Add("Error: Target directory not set.");
            }
            if (errors.Count != 0)
            {
                ErrorLog myLog = new ErrorLog();
                foreach (string error in errors)
                {
                    myLog.addError(error);
                }
                errors.Clear();
                myLog.ShowDialog();
            }
        }

        private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.progressBar1.Value = 0; 
            lBoxFiles.Items.Clear();
            this.btnStart.Enabled = true;
            if (fromContext == true) 
            {
                Application.Exit();
            }
        }

        private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressBar1.Value += e.ProgressPercentage;
        }

        private void openFilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult fileResult = openFileDialog1.ShowDialog();
            if (fileResult != DialogResult.Cancel) 
            {
                foreach (string fileName in openFileDialog1.FileNames)
                {
                    addJamZip(fileName);
                }
            }
        }

        private void openFilesFromFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult folderResult = folderBrowserDialog1.ShowDialog();
            if (folderResult != DialogResult.Cancel) 
            {
                string folderName = folderBrowserDialog1.SelectedPath;
                string[] files = Directory.GetFiles(folderName);
                foreach (string fileName in files)
                {
                    addJamZip(fileName);
                }
            }
        }

        private void exitToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            Application.Exit();
        }

    }
}
